#include "Sprite.h"
// #define TIXML_USE_STL
#include "tinyxml.h"
#include "gl/glpng.h"

void 
Frame::Draw(int x, int y,float rot, float sx, float sy,float a,float r, float g, float b,bool flipLR, bool flipUD) const
{
		glColor4f(r,g,b,a);
		glBindTexture(GL_TEXTURE_2D, texID);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
		glEnable(GL_BLEND);

		glEnable(GL_TEXTURE_2D);

		glPushMatrix();
		glTranslatef((float)x,(float)y,0);

		glRotatef(rot,0,0,1);
		
		glScalef(sx,sy,1);

		if( flipLR )
			glTranslatef(-(float)(size[0]-offset[0]),-(float)offset[1],0);
		else
			glTranslatef(-(float)offset[0],-(float)offset[1],0);
		
		glBegin(GL_QUADS);
		glNormal3f(0,0,1);
			
			glTexCoord2f(flipLR ? tcLR[0] : tcUL[0], flipUD ? tcLR[1] : tcUL[1]);
			glVertex2i(0,0);

			glTexCoord2f(flipLR ? tcUL[0] : tcLR[0],flipUD ? tcLR[1] : tcUL[1]);
			glVertex2i(size[0],0);

			glTexCoord2f(flipLR ? tcUL[0] : tcLR[0],flipUD ? tcUL[1] : tcLR[1]);
			glVertex2i(size[0],size[1]);

			glTexCoord2f(flipLR ? tcLR[0] : tcUL[0],flipUD ? tcUL[1] : tcLR[1]);
			glVertex2i(0,size[1]);
		glEnd();
		glPopMatrix();

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);

}

void 
Frame::DrawPercent(int x, int y, float rot, float sx, float sy,float a,float r, float g, float b, float percent) const
{
		glColor4f(r,g,b,a);
		glBindTexture(GL_TEXTURE_2D, texID);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
		glEnable(GL_BLEND);

		glEnable(GL_TEXTURE_2D);

		glPushMatrix();
		glTranslatef((float)x,(float)y,0);

		glRotatef(rot,0,0,1);
		
		glScalef(sx,sy,1);
		glTranslatef(-(float)offset[0],-(float)offset[1],0);
		

		float peUL[2];
		peUL[0] = tcUL[0];
		peUL[1] = percent*(tcUL[1] - tcLR[1]) + tcLR[1];
		float sizePer = (1 - percent) * size[1];


		glBegin(GL_QUADS);
		glNormal3f(0,0,1);
			
			glTexCoord2f(peUL[0], peUL[1]);
			glVertex2i(0,sizePer);

			glTexCoord2f(tcLR[0],peUL[1]);
			glVertex2i(size[0],sizePer);

			glTexCoord2f(tcLR[0],tcLR[1]);
			glVertex2i(size[0],size[1]);

			glTexCoord2f(peUL[0],tcLR[1]);
			glVertex2i(0,size[1]);
		glEnd();
		glPopMatrix();

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);

}

bool 
FrameSet::LoadFromXML(const string & fname)
{
	TiXmlDocument doc;
	if (!doc.LoadFile(fname.c_str()))
		return false;
	TiXmlHandle docHandle( &doc );
	TiXmlElement * root = docHandle.FirstChild("FrameSet").Element();
	if (!root)
		return false;

	
	
	TiXmlNode *	tex = root->FirstChild("Texture");
	while (tex)
	{
		const char * name = tex->ToElement()->Attribute("name");
		if (!name)
			return false;
		
		pngInfo info;
		GLuint id;

		//load the texture
		glGenTextures(1, &id);
		glBindTexture(GL_TEXTURE_2D, id);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

		if (!pngLoad(name, PNG_NOMIPMAP, PNG_ALPHA, &info)) 
		{
			glDeleteTextures(1,&id);
			return false; //can't load
		}
		ids.push_back(id);

		bool usealpha = info.Alpha > 0;

		//now load all of the frames
		TiXmlNode *	frame = tex->FirstChild("Frame");
		while (frame)
		{
			Frame *fr = new Frame();
			fr->alpha = usealpha;
			fr->texID = id;
			fr->offset[0] = fr->offset[1] = 0;
			fr->size[0] = fr->size[1] = 0;

			double x,y,w,h;
			x = y = w = h = 0;
			frame->ToElement()->Attribute("x",&x);
			frame->ToElement()->Attribute("y",&y);
			frame->ToElement()->Attribute("w",&w);
			frame->ToElement()->Attribute("h",&h);
			frame->ToElement()->Attribute("xo",&fr->offset[0]);
			frame->ToElement()->Attribute("yo",&fr->offset[1]);

			fr->tcUL[0] = (float)x / info.Width;
			fr->tcUL[1] = (float)y / info.Height;
			fr->tcLR[0] = (float)(x +w)/ info.Width;
			fr->tcLR[1] = (float)(y +h)/ info.Height;
			fr->size[0] = (unsigned int)w;
			fr->size[1] = (unsigned int)h;

			const char * framename = frame->ToElement()->Attribute("name");
			fr->name = framename;

			map<string,Frame*>::iterator i = frames.find(framename);
			if (i != frames.end())
			{
				delete fr;
				return false;
			}

			frames[framename] = fr;



			frame = tex->IterateChildren("Frame", frame);
		}

		tex = root->IterateChildren("Texture", tex);
	}
	
	return true;

}

const Frame * 
FrameSet::GetFrame(const string & name) const
{
	map<string,Frame*>::const_iterator i = frames.find(name);
	if (i == frames.end())
		return NULL;
	else
		return i->second;
}

const Animation * 
AnimationSet::GetAnimation(const string & name) const
{
	map<string,Animation*>::const_iterator i = anims.find(name);
	if (i == anims.end())
		return NULL;
	else
		return i->second;
}

FrameSet::~FrameSet()
{
	for ( map<string,Frame*>::iterator i = frames.begin(); i != frames.end(); i++)
		delete i->second;
	for (unsigned int j = 0; j < ids.size(); j++)
		glDeleteTextures(1,&ids[j]);

}

AnimationSet::~AnimationSet()
{
	for ( map<string,Animation*>::iterator i = anims.begin(); i != anims.end(); i++)
		delete i->second;
}


bool 
AnimationSet::LoadFromXML(const string & fname)
{
	TiXmlDocument doc;
	if (!doc.LoadFile(fname.c_str()))
		return false;
	TiXmlHandle docHandle( &doc );
	TiXmlElement * root = docHandle.FirstChild("AnimSet").Element();
	if (!root)
		return false;


	
	TiXmlNode *	tex = root->FirstChild("Animation");
	while (tex)
	{
		const char * name = tex->ToElement()->Attribute("name");
		if (!name)
			return false;
		
		Animation * anim = new Animation();

		//now load all of the frames
		TiXmlNode *	frame = tex->FirstChild("f");
		while (frame)
		{
			double t = 0;
			frame->ToElement()->Attribute("t",&t);
			const char * framename = frame->ToElement()->Attribute("f");
			if (!framename)
			{
				delete anim;
				return false;
			}
			anim->times.push_back(t);
			anim->frameNames.push_back(framename);
			anim->length += t;
			const Frame *fptr = frameset.GetFrame(framename);
			if (!fptr)
			{
				delete anim;
				return false;
			}
			anim->frames.push_back(fptr);
			frame = tex->IterateChildren("f", frame);
		}

		map<string,Animation*>::iterator i = anims.find(name);
		if (i != anims.end())
		{
			delete anim;
			return false;
		}
		anims[name] = anim;
		tex = root->IterateChildren("Animation", tex);
	}
	
	return true;

}
/*
string
Animation::GetFrameName(double time) const
{
	unsigned int numFrames = frameNames.size();
	if (numFrames == 0)
		return "";
	if (time > length)
		return frameNames[numFrames-1];
	else
	{
		double t = 0;
		unsigned int index = 0;
		while (index < numFrames)
		{
			t += times[index];
			if (t >= time)
				return frameNames[index];
			index++;
		}
		return frameNames[numFrames-1];
	}
}

*/
const Frame *
Animation::GetFrame(double time) const
{
	unsigned int numFrames = frames.size();
	if (numFrames == 0)
		return NULL;
	if (time > length)
		return frames[numFrames-1];
	else
	{
		double t = 0;
		unsigned int index = 0;
		while (index < numFrames)
		{
			t += times[index];
			if (t >= time)
				return frames[index];
			index++;
		}
		return frames[numFrames-1];
	}
}

bool 
AnimationSet::GetFrame(const string & name, double time, const Frame * & frame) const
{
	const Animation * anim = GetAnimation(name);
	if (!anim)
	{
		frame = NULL;
		return false;
	}
	frame = anim->GetFrame(time);
	return time < anim->GetLength();

}

bool 
AnimationSet::GetFrame(const string & name, const Frame * & frame) const
{
	frame = frameset.GetFrame(name);
	return frame != NULL;
}

void 
Sprite::IncrementTimer(double dt)
{
	currentTime += dt;
	stale = true;
}

bool 
Sprite::SetFrame(const string & fname)
{
	Frame * f;
	if (!animset.GetFrame(fname,f))
		return false;
	lastFrame = f;
	playing = false;
	stale = false;

	return true;

}
void 
Sprite::SetNextAnimation(const string & name)
{
	nextAnim = animset.GetAnimation(name);
	nextAnimName = name;
}

void Sprite::ClearNextAnimation()
{
	nextAnim = 0;
}

bool 
Sprite::PlayAnimation(const string & fname)
{
	const Animation * an;
	if ((an = animset.GetAnimation(fname)) == NULL)
		return false;
	animName = fname;
	currentAnim = an;
	currentTime = 0;
	stale = true;
	playing = true;
	return true;

}

void 
Sprite::SetPos(float x, float y)
{
	pos[0] = x;
	pos[1] = y;
}

void
Sprite::SetAlpha(float a)
{
	alpha = a;
}
void 
Sprite::SetRot(float r)
{
	rot = r;

}

void
Sprite::SetColour(float r, float g, float b)
{
	colour[0] = r;
	colour[1] = g;
	colour[2] = b;

}
void 
Sprite::SetScale(float sx, float sy)
{
	scale[0] = sx;
	scale[1] = sy;
}

void 
Sprite::Draw()
{
	if (playing && stale)
	{
		if (!currentAnim)
			return;
		
		lastFrame = currentAnim->GetFrame(currentTime);
		//string framename = currentAnim->GetFrameName(currentTime);
		//animset.GetFrame(framename,lastFrame);
		if (currentTime > currentAnim->GetLength())
		{
			if (nextAnim)
			{
				currentTime = currentTime - currentAnim->GetLength();
				currentAnim = nextAnim;
				animName = nextAnimName;
			}
			else
				playing = false;
		}
	}
	
	glPushMatrix();
	
	if (lastFrame)
		lastFrame->Draw((int)pos[0],(int)pos[1],rot,scale[0],scale[1],alpha,colour[0],colour[1],colour[2],flipLR,flipUD);

	glPopMatrix();
	stale = false;
}

