#include <windows.h>
#include <GL/gl.h>

#include "bbox.h"


BBox::BBox()
{
	center[0] = center[1] = width = height = 0.0;
}

BBox::BBox(float w, float h)
{
	center[0] = center[1] = 0.0;
	width  = w;
	height = h;
}

BBox::BBox(float cx, float cy, float w, float h)
{
	center[0] = cx;
	center[1] = cy;
	width     = w;
	height    = h;
}

BBox::BBox(const BBox& box)
{
	center[0] = box.center[0];
	center[1] = box.center[1];
	width     = box.width;
	height    = box.height;
}

BBox::~BBox()
{
}

BBox&
BBox::operator=(const BBox& box)
{
	center[0] = box.center[0];
	center[1] = box.center[1];
	width     = box.width;
	height    = box.height;

	return *this;
}

void
BBox::getCenter(float& x, float& y) const
{
	x = center[0];
	y = center[1];
}

void
BBox::setCenter(float  x, float  y)
{
	center[0] = x;
	center[1] = y;
}

float
BBox::getWidth() const
{
	return width;
}

void
BBox::setWidth(float w)
{
	width = w;
}

float
BBox::getHeight() const
{
	return height;
}

void
BBox::setHeight(float h)
{
	height = h;
}

void
BBox::getMin(float& x, float& y) const
{
	x = center[0] - 0.5f * width;
	y = center[1] - 0.5f * height;
}

void
BBox::getMax(float& x, float& y) const
{
	x = center[0] + 0.5f * width;
	y = center[1] + 0.5f * height;
}

void
BBox::grow(const BBox& box)
{
	float xmin, xmax, bxmin, bxmax;
	float ymin, ymax, bymin, bymax;

	box.getMin(bxmin, bymin);
	box.getMax(bxmax, bymax);

	getMin(xmin, ymin);
	getMax(xmax, ymax);

	if( bxmin < xmin ) xmin = bxmin;
	if( bxmax > xmax ) xmax = bxmax;
	if( bymin < ymin ) ymin = bymin;
	if( bymax > ymax ) ymax = bymax;

	width  = xmax - xmin;
	height = ymax - ymin;

	center[0] = xmin + 0.5f * width;
	center[1] = ymin + 0.5f * height;
}

bool
BBox::pointInBox(float x, float y) const
{
	float xmin, xmax;
	float ymin, ymax;

	getMin(xmin, ymin);
	getMax(xmax, ymax);

	if( (x > xmin) && (x < xmax) && (y > ymin) && (y < ymax) )
		return true;

	return false;
}

void
BBox::draw() const
{
	float xmin, xmax;
	float ymin, ymax;

	getMin(xmin, ymin);
	getMax(xmax, ymax);

	glColor3f(0.5, 0.5, 0.5);
	glBegin(GL_LINE_LOOP);
		glVertex2f(xmin, ymin);
		glVertex2f(xmin, ymax);
		glVertex2f(xmax, ymax);
		glVertex2f(xmax, ymin);
	glEnd();
}
