#include "clouds.h"
#include "util.h"

static float windD[2] = {0.2f, 0.8f};

const int WIDTH = 1024;
const int HEIGHT = 768;

const int cWidth = 256;
const int cHeight = 256;

Clouds::Clouds()
{
	const int numClouds = 15;

	// Load images
	frames.LoadFromXML("art/cloudFrames.xml");

	const int numFrames = 2;
	const Frame* f[numFrames];
	f[0] = frames.GetFrame("cloud1");
	f[1] = frames.GetFrame("cloud2");

	// Create clouds
	for (int i = 0; i < numClouds; ++i)
	{
		int frame = rand() % 2;
		float str = 50.0*randf() + 30.0;
		Cloud c = {randf() * (WIDTH + cWidth) - cWidth/2.0, 
			       randf() * (HEIGHT + cHeight) - cHeight/2.0,
				   0, str*windD[0], str*windD[1], 0, f[frame]};
		clouds.push_back(c);
	}
}


void Clouds::Draw()
{
	// const Frame* f = frames.GetFrame("cloud1");
	// f->Draw(500,500,0,1.0,1.0,0.3,1.0,1.0,1.0);
	for (CloudIt it = clouds.begin(); it != clouds.end(); ++it)
	{
		it->f->Draw(it->pos[0],it->pos[1],0,2.0,2.0,0.3,1.0,1.0,1.0,false,false);
	}
}


void Clouds::Update(double timeElapsed)
{
	const double buf = 1.2;

	for (CloudIt it = clouds.begin(); it != clouds.end(); ++it)
	{
		Cloud& c = *it;
		c.pos[0] += c.vel[0] * timeElapsed;
		c.pos[1] += c.vel[1] * timeElapsed;
		c.pos[2] += c.vel[2] * timeElapsed;

		// Check if off screen
		if (c.pos[0] < -  cWidth*buf || c.pos[0] > WIDTH  + cWidth*buf ||
			c.pos[1] < - cHeight*buf || c.pos[1] > HEIGHT + cHeight*buf)
		{
			// Construct new frame

			bool border = rand() % 2;
			if (border)
			{
				c.pos[0] = (c.vel[0] > 0) ? -cWidth : WIDTH + cWidth;
				c.pos[1] = randf() * (HEIGHT + cHeight) - cHeight/2.0;
			} else {
				c.pos[0] = randf() * (WIDTH + cWidth) - cWidth/2.0;
				c.pos[1] = (c.vel[1] > 0) ? -cHeight : HEIGHT + cHeight;
			}

		}
	}
}
