#include "spellSource.h"

#include "particle.h"
#include "SoundManager.h"

const float sizeIncrease = 1.3;

SpellSource::SpellSource(float x, float y) :
	Entity(x, y),
	lifetime(MIN_SS_LIFE)
{
}

SpellSource::~SpellSource()
{
}

SpellSource*
SpellSource::createSpellSource(eSpellSources type, float x, float y, bool useDefault, int w, int s)
{
	switch( type )
	{
		case eFire:
		case eCreateFire:
		{
			if( useDefault )
				return new FireSource(x, y);
			else
				return new FireSource(x, y, w, s);
			break;
		}

		case eWater:
		case eCreateWater:
		{
			if( useDefault )
				return new WaterSource(x, y);
			else
				return new WaterSource(x, y, w, s);
			break;
		}

		case eEarth:
		case eCreateEarth:
		{
			if( useDefault )
				return new EarthSource(x, y);
			else
				return new EarthSource(x, y, w, s);
			break;
		}

		case eElec:
		case eCreateElec:
		{
			if( useDefault )
				return new ElecSource(x, y);
			else
				return new ElecSource(x, y, w, s);
			break;
		}

		default:
		{
			return 0;
			break;
		}
	}
}

float
SpellSource::getLifetime() const
{
	return lifetime;
}

void
SpellSource::setLifetime(float t)
{
	lifetime = t;

	if( lifetime < MIN_SS_LIFE ) lifetime = MIN_SS_LIFE;
	if( lifetime > MAX_SS_LIFE ) lifetime = MAX_SS_LIFE;
}

float
SpellSource::decLifetime(float dt)
{
	lifetime -= dt;

	if( lifetime < MIN_SS_LIFE ) lifetime = MIN_SS_LIFE;
	if( lifetime > MAX_SS_LIFE ) lifetime = MAX_SS_LIFE;

	return lifetime;
}

void
SpellSource::resetLifetime()
{
	lifetime = MAX_SS_LIFE;
}

bool
SpellSource::isDead() const
{
	return (!(lifetime > MIN_SS_LIFE));
}

SpellPoints&
SpellSource::getWizardSP()
{
	return dWizard;
}

SpellPoints&
SpellSource::getShotSP()
{
	return dSpellShot;
}



eSpellReaction SpellSource::affectShot(SpellShot& s) const
{
	char sound[256];
	sprintf(sound, "%sshot", eSpellSourcesStr[type()]);

	if (s.type == eGeneric)
	{
		// Change emitter to specific shot
		s.points += dSpellShot;
		ParticleEmitter* em = shotEmitter[type()]->Clone();
		emitterSet.Add(em);
		s.ResetEmitter(em);
		s.frame = shotFrame[type()];
		s.type = type();
		PlaySoundEffect(sound);
	}
	if (s.type == type())
	{
		s.points += dSpellShot;
		s.size *= sizeIncrease;
		PlaySoundEffect(sound);
	}
	return eNone;
}



eSpellReaction FireSource::affectShot(SpellShot& s) const 
{
	SpellSource::affectShot(s);
	if (s.type == eWater)
		return eShotFizzle;
	else
		return eNone;
}


eSpellReaction WaterSource::affectShot(SpellShot& s) const 
{
	SpellSource::affectShot(s);
	if (s.type == eFire)
		return eShotFizzle;
	else
		return eNone;
}


eSpellReaction EarthSource::affectShot(SpellShot& s) const 
{
	SpellSource::affectShot(s);
	if (s.type == eElec)
		return eShotFizzle;
	else
		return eNone;
}

eSpellReaction ElecSource::affectShot(SpellShot& s) const 
{
	SpellSource::affectShot(s);
	if (s.type == eEarth)
		return eShotFizzle;
	else
		return eNone;
}
